/**
 * 模块说明
 * @module 基础组件描述
 * @author Max.Law 2022/08/26
 */

export default {
  'form-title': {
    name: '主要标题',
    type: 'form-title',
    grade: 'brick',
    icon: 'el-icon-star-off',
    title: '标题',
    textAlign: 'center',
    formItem: {
      span: 24,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  'group-title': {
    name: '组标题',
    type: 'group-title',
    title: '组标题',
    grade: 'brick',
    icon: 'el-icon-star-on',
    formItem: {
      span: 24,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  row: {
    name: '行容器',
    type: 'row',
    grade: 'frame',
    gutter: 0,
    icon: 'el-icon-crop',
    column: [],
  },
  'group-container': {
    name: '组容器',
    type: 'group-container',
    grade: 'frame',
    icon: 'el-icon-full-screen',
    title: '组容器',
    column: [],
  },
  button: {
    name: '按钮',
    type: 'button',
    grade: 'brick',
    icon: 'el-icon-open',
    nativeType: 'button',
    html: '按钮',
    formItem: {
      span: 3,
      noLabel: true,
      labelWidth: '0px',
    },
    events: ['click'],
  },
  'button-group': {
    name: '按钮组',
    type: 'button-group',
    grade: 'brick',
    icon: 'el-icon-open',
    buttonList: [{ nativeType: 'button', html: '按钮' }],
    formItem: {
      span: 3,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  link: {
    name: '文字链接',
    type: 'link',
    grade: 'brick',
    attrType: 'primary',
    icon: 'el-icon-link',
    html: '文字链接',
    formItem: {
      span: 4,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  alert: {
    name: '提示语',
    type: 'alert',
    grade: 'brick',
    icon: 'el-icon-s-opportunity',
    title: '文字提示',
    attrType: 'error',
    effect: 'light',
    events: ['close'],
    formItem: {
      span: 11,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  'wang-editor': {
    name: '富文本',
    type: 'wang-editor',
    grade: 'brick',
    icon: 'el-icon-brush',
    placeholder: '',
    formItem: {
      span: 16,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  'form-dialog': {
    name: '对话框',
    type: 'form-dialog',
    grade: 'frame',
    icon: 'el-icon-service',
    title: '对话框',
    visible: false,
    events: ['open', 'close', 'confirm'],
    column: [],
  },
  input: {
    name: '输入框',
    type: 'input',
    grade: 'lego',
    icon: 'el-icon-edit',
    maxlength: 50,
    minlength: 1,
    autocomplete: 'off',
    events: ['blur', 'focus', 'change', 'input', 'clear'],
  },
  autocomplete: {
    name: '带建议输入',
    type: 'autocomplete',
    grade: 'lego',
    icon: 'el-icon-s-help',
    maxlength: 50,
    debounce: 300,
    events: ['select', 'change'],
  },
  textarea: {
    name: '文本框',
    type: 'input',
    grade: 'lego',
    icon: 'el-icon-tickets',
    attrType: 'textarea',
    maxlength: 500,
    rows: 4,
    events: ['blur', 'focus', 'change', 'input', 'clear'],
  },
  'form-select': {
    name: '选择框',
    type: 'form-select',
    grade: 'lego',
    icon: 'el-icon-news',
    autocomplete: 'off',
    remoteData: {},
    events: ['blur', 'focus', 'change', 'visibleChange', 'removeTag', 'clear'],
  },
  cascader: {
    name: '级联选择',
    type: 'cascader',
    grade: 'lego',
    icon: 'el-icon-office-building',
    remoteData: {},
    props: {
      expandTrigger: 'click',
    },
    events: ['blur', 'focus', 'change', 'visibleChange', 'removeTag'],
  },
  checkbox: {
    name: '多选框',
    type: 'checkbox',
    grade: 'lego',
    html: '单一选项',
    icon: 'el-icon-circle-check',
    events: ['change'],
    size: 'small',
    formItem: {
      span: 4,
    },
  },
  'checkbox-group': {
    name: '多选框组',
    type: 'checkbox-group',
    grade: 'lego',
    icon: 'el-icon-circle-check',
    remoteData: {},
    events: ['change'],
  },
  radio: {
    name: '单选框',
    type: 'radio',
    grade: 'lego',
    html: '单一选项',
    icon: 'el-icon-success',
    events: ['change'],
    size: 'small',
    formItem: {
      span: 5,
    },
  },
  'radio-group': {
    name: '单选框组',
    type: 'radio-group',
    grade: 'lego',
    icon: 'el-icon-success',
    remoteData: {},
    events: ['change'],
  },
  'date-picker': {
    name: '日期选择',
    type: 'date-picker',
    grade: 'lego',
    icon: 'el-icon-date',
    attrType: 'date',
    format: 'yyyy-MM-dd',
    valueFormat: 'yyyy-MM-dd',
    events: ['change', 'blur', 'focus'],
  },
  'time-select': {
    name: '固定时间',
    type: 'time-select',
    grade: 'lego',
    icon: 'el-icon-time',
    events: ['change', 'blur', 'focus'],
  },
  'time-picker': {
    name: '任意时间',
    type: 'time-picker',
    grade: 'lego',
    icon: 'el-icon-time',
    valueFormat: 'HH:mm:ss',
    events: ['change', 'blur', 'focus'],
  },
  'form-upload': {
    name: '上传',
    type: 'form-upload',
    grade: 'lego',
    icon: 'el-icon-upload2',
    events: ['onProgress'],
  },
  'form-download': {
    name: '下载列表',
    type: 'form-download',
    grade: 'lego',
    icon: 'el-icon-download',
  },
  'display-table': {
    name: '展示表格',
    type: 'display-table',
    grade: 'brick',
    icon: 'el-icon-menu',
    size: 'small',
    optionList: [{ label: '列一', prop: '', align: 'left' }],
    events: ['rowClick'],
    formItem: {
      span: 24,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  descriptions: {
    name: '描述列表',
    type: 'descriptions',
    grade: 'brick',
    icon: 'el-icon-s-grid',
    direction: 'horizontal',
    size: 'small',
    column: 3,
    optionList: [{ label: '用户名', prop: '' }],
    formItem: {
      span: 24,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  'form-table': {
    name: '表格',
    type: 'form-table',
    grade: 'frame',
    icon: 'el-icon-tickets',
    attrType: 'index',
    size: 'small',
    column: [],
    events: ['rowClick'],
    formItem: {
      span: 24,
    },
  },
  'picture-frame': {
    name: '相框',
    type: 'picture-frame',
    grade: 'lego',
    icon: 'el-icon-picture',
    formItem: {
      span: 6,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  communication: {
    name: '自通信',
    type: 'communication',
    grade: 'lego',
    formatType: 'tel',
    icon: 'el-icon-s-promotion',
  },
  iframe: {
    name: 'iframe',
    type: 'iframe',
    grade: 'brick',
    icon: 'el-icon-monitor',
    formItem: {
      span: 24,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  'input-number': {
    name: '计数器',
    type: 'input-number',
    grade: 'lego',
    icon: 'el-icon-circle-plus-outline',
    events: ['change', 'blur', 'focus'],
  },
  switch: {
    name: '开关',
    type: 'switch',
    grade: 'lego',
    with: 40,
    icon: 'el-icon-open',
    events: ['change'],
  },
  'color-picker': {
    name: '颜色选择器',
    type: 'color-picker',
    grade: 'lego',
    icon: 'el-icon-s-open',
    events: ['change', 'active-change'],
  },
  'annex-list': {
    name: '附件列表',
    type: 'annex-list',
    grade: 'brick',
    icon: 'el-icon-upload',
    size: 'mini',
    remoteData: {},
    formItem: {
      span: 24,
      noLabel: true,
      labelWidth: '0px',
    },
  },
  'form-address': {
    name: '省市区',
    type: 'form-address',
    grade: 'lego',
    icon: 'el-icon-location-outline',
    formItem: {
      label: '省市区',
    },
  },
};
